
input[type="checkbox"] {
 width:22px; 
 height:22px; 
 color:yellow;
 border:4px outset green;
 background-color:green;
}

input[type="radio"] {
 width:17px; 
 height:17px; 
 border:3px inset;
 background-color:green;
 color:yellow;
 -moz-border-radius:0;
}

:-moz-radio {
 width:8px;
 width:8px;
 background-color:yellow;
 color:yellow;
 -moz-border-radius:0;
}

input[type=button] {
 border:4px outset green;
 -moz-border-radius:8px;
 color:black;
 background-color:green;
 padding-left:10px;
 padding-right:10px;
 font-weight:bold;
}

input[type=button]:active {
 border:4px inset green;
 padding-left:12px;
 padding-top:2px;
 padding-right:8px;
}

select {
  -moz-border-radius:0px;
  border:4px inset green;
  background-color:#0000cc;
}

:-moz-dropdown-list {
  background-color:#0000cc;
  border:4px inset green;
}

option {
 padding-right:10px;
 padding-left:10px;
 font-weight:bold;
 background-color:green;
}

option[_moz-option-selected] {
  color:red;
  background-color:yellow;
}

:-moz-dropdown-btn-out {
  background-color:green;
  border: 2px outset green;
}

:-moz-dropdown-btn-pressed {
  background-color:green;
  border: 2px inset green;
  color:yellow;
}

:combobox-textselected {
  background-color:green;
  color:yellow;
}

:combobox-text {
  background-color:green;
  color:yellow;
}

optgroup { 
 background-color:#0000cc;
 color:white;
}

optgroup > option {
  background-color:green;
  color:black;
}
  
